<?php

namespace MotoStore\Extentions\Creditreform;

use Moto;
use MotoStore;
use MotoStore\Settings\Entity\Option;
use MotoStore\Content\Content\DI\SMStatic;
use MotoStore\Settings\Repository\OptionRepository;
use SoapClient;
use SoapVar;
use SoapFault;
use Exception;
use WsseAuthHeader;
use stdClass;

class Creditreform
{
    public static function check($customer_details){
        $optRepository = clone SMStatic::getInstance()
            ->get ('MotoStore\EntityManager')
            ->getRepository ('MotoStore\Settings\Entity\Option');
        $enabled =  $optRepository->getOption (Option::OPTION_STORE_SETTINGS_EXTENTIONS, 'enable_creditreform');
        $username =  $optRepository->getOption (Option::OPTION_STORE_SETTINGS_EXTENTIONS, 'creditreform_username');
        $password =  $optRepository->getOption (Option::OPTION_STORE_SETTINGS_EXTENTIONS, 'creditreform_password');
        if (!$enabled || empty($enabled) || !$username || empty($username) || !$password || empty($password))
            return false;
        try {
            $client = new SoapClient(Moto\System::getAbsoluteUrl('@plugins/moto-store-plugin/extensions/credit_reform_checker/CrediConnect_V2.1.5.wsdl'), array('trace' => 1));
            $wsse_header = new WsseAuthHeader($username, $password);
            $client->__setSoapHeaders(array($wsse_header));
        }
        catch (Exception $e) {
            error_log($e->getMessage());
            return true;
        }

        try{
            $ns = 'http://crediconnect.ch/V2/01/';
            $ns2 = 'http://schemas.creditreform.ch/V2/01';
            $productCode = new SoapVar('2901CMSCOR', XSD_STRING, null,$ns);
            $address = new stdClass();

            $customer_first_name = isset($customer_details['first_name'])?$customer_details['first_name']:'';
            $customer_name = isset($customer_details['last_name'])?$customer_details['last_name']:'';
            $customer_city = isset($customer_details['city'])?$customer_details['city']:'';
            $customer_zip = isset($customer_details['post_code'])?$customer_details['post_code']:'';

            $address->AddressType = new SoapVar('Void',XSD_STRING, null, $ns2, null,$ns2 );
            $address->FirstName = new SoapVar($customer_first_name,XSD_STRING, null, $ns2, null,$ns2);
            $address->Name = new SoapVar($customer_name,XSD_STRING, null, $ns2, null,$ns2);
            $address->PostalCode = new SoapVar($customer_zip, XSD_STRING, null, $ns2, null,$ns2);
            $address->City = new SoapVar($customer_city, XSD_STRING, null, $ns2, null,$ns2);
            $address->Country = new SoapVar('CH', XSD_STRING, null, $ns2, null,$ns2);

            $searchFilter = new StdClass();
            $searchFilter->ArchiveType = new SoapVar('MainArchive',XSD_STRING, null, $ns2, null,$ns2);
            $searchFilter->MatchLevel = new SoapVar('Low',XSD_STRING, null, $ns2, null,$ns2);

            $reasonOfInquiry = new SoapVar('Creditworthiness', XSD_STRING, null, null);
            $request = array();
            $request[] = new SoapVar($productCode, SOAP_ENC_OBJECT, NULL, NULL, 'productCode',$ns);
            $request[] = new SoapVar($address, SOAP_ENC_OBJECT,  NULL, NULL, 'searchedAddress', $ns);
            $request[] = new SoapVar($searchFilter, SOAP_ENC_OBJECT,  NULL, NULL, 'searchFilter', $ns);
            $request[] = new SoapVar($reasonOfInquiry, SOAP_ENC_OBJECT, NULL, NULL, 'reasonOfInquiry',$ns);

            $matchResponse = $client->Match(new SoapVar($request, SOAP_ENC_OBJECT));
            if ($matchResponse->MatchResult->CrefoID && $matchResponse->MatchResult->ReportContent->KeyValuePairOfstringstring->value && $matchResponse->MatchResult->ReportContent->KeyValuePairOfstringstring->value == 'Green'){
                return false;
            }

        }
        catch(SoapFault $f){
            error_log('error soap request '.json_encode($f->getMessage()));
            return true;
        }
        catch (Exception $e) {
            error_log('error soap request '.json_encode($e->getMessage()));
            return true;
        }
        return true;
    }
}