<?php

/***********************************************************************************************************************
 * @author: <for.samohin@gmail.com>
 **********************************************************************************************************************/
namespace MotoStore\Payment\Barclaycard;

use MotoStore\Content\Helper\Link;
use MotoStore\Order\Entity\Order;
use MotoStore\Order\Entity\Transaction;
use MotoStore\Payment\Gateway\PaymentModule;
use Zend\Http\PhpEnvironment\Request;

class Barclaycard extends PaymentModule
{
    public function getName()
    {
        return 'Barclaycard';
    }

    public function getImage()
    {
        return 'barclaycard.png';
    }

    public function getDescription()
    {
        return 'Url for notifications: '. Link::storeLink ('checkout', 'success', array ('provider' => $this->getName ()), false) ;
    }

    public function getHelpText()
    {
        return 'Url for notifications: '. Link::storeLink ('checkout', 'success', array ('provider' => $this->getName ()), false) ;
    }

    public function setVariables()
    {
        $this
            ->addVariable (
                self::OPTION_DISPLAY_NAME,
                array(
                    'name' => 'Display Name',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => 'Barclaycard',
                    'required' => true,
                )
            )->addVariable (
                'pspid',
                array(
                    'name' => 'PSPID',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                'secret_key',
                array(
                    'name' => 'Pass Phrase',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                'test_mode',
                array(
                    'name' => 'Test Mode',
                    'type' => self::VARIABLE_TYPE_SELECT,
                    'help' => '',
                    'default' => 0,
                    'required' => true,
                    'options' => array (
                        0 => 'No',
                        1 => 'Yes'
                    ),
                )
            )->addVariable (
                self::OPTION_ENABLED,
                array(
                    'name' => 'Enabled',
                    'type' => self::VARIABLE_TYPE_SELECT,
                    'help' => '',
                    'default' => 0,
                    'required' => true,
                    'options' => array (
                        0 => 'No',
                        1 => 'Yes'
                    ),
                )
            );
            
    }
    /**
     * @param Order $order
     * @return mixed|void
     */
    public function checkout(Order $order)
    {


        if ($this->getOption ('test_mode')){
            $api_url = 'https://mdepayments.epdq.co.uk/ncol/test/orderstandard_utf8.asp';
        } else {
            $api_url = 'https://mdepayments.epdq.co.uk/ncol/prod/orderstandard_utf8.asp';
        }
        $currency = $this->getCurrency();

        $names = array ();

        foreach ($order->getProducts () as $product)
        {
            $names [] = $product->getName ();
        }
        $data = array(
            "PSPID"  => $this->getOption ('pspid'),
            "ORDERID" => $order->getOrderId(),
            "AMOUNT" => $order->getTotalPrice () * $currency->getExchangeRate (),
            "CURRENCY" => $currency->getCode (),
            "LANGUAGE" => 'EN',
            "ACCEPTURL" => Link::storeLink ('checkout', 'success', array ('provider' => $this->getName ()), false),
            "CANCELURL" => Link::storeLink ('cart', null, array(), false),
            "TITLE" => 'Order #' . $order->getOrderId () . ' [' .  join (', ', $names) . ']',
        );
        $sha_sign = $this->getSha($data);
        $data ['SHASIGN'] = $sha_sign;

        echo $this->form($api_url, $data);


    }
    public function getSha($data){
        $data = asort($data);
        $secret = $this->getOption ('secret_key');
        $string_to_hash = '';
        foreach ($data as $key=>$value){
            if($key == 'SHASIGN'){

            } else {
                $string_to_hash .= $key.'='.$value.$secret;
            }
        }
        return hash('sha512', $string_to_hash);
    }

    public function handle(Request $request)
    {
        if (isset($_REQUEST['ORDERID'])){
            $orderid = $_REQUEST['ORDERID'];
            $trans = $_REQUEST['PAYID'];
            $order = $this->getCurrentOrder ($orderid);
            if (!$order){
                die('Order not found');
            }
            if ($_REQUEST['STATUS'] == 9){
                $em = $this->serviceLocator->get ('MotoStore\EntityManager');
                $status = Transaction::PAYMENT_STATUS_PAID;
                $orderStatus = Order::ORDER_PAYMENT_ACCEPTED;
                $transaction = new Transaction ();
                $transaction->setOrder ($order);
                $transaction->setTransactionId ($trans);
                $date = new \DateTime ();
                $transaction->setDate ($date);
                $transaction->setPaymentMethod($this->getName ());
                $transaction->setStatus ($status);
                $transaction->setAdditional(json_encode ($_REQUEST));
                $em->persist ($transaction);
                $order->addTransaction ($transaction);
                $order->setStatus ($orderStatus);
                $em->merge ($order);
                $em->flush ();
                $this->getCart ()->setIsSuccess (true);
                \Moto\Hook::trigger("MOTOSTORE_HOOK_ORDER_PAID",$order->getOrderId());
            }

        }
    }
}