<?php

/***********************************************************************************************************************
 * @author: <for.samohin@gmail.com>
 **********************************************************************************************************************/
namespace MotoStore\Payment\BraintreePayment;

use MotoStore\Content\Helper\Link;
use MotoStore\Order\Entity\Order;
use MotoStore\Order\Entity\Product;
use MotoStore\Order\Entity\Transaction;
use MotoStore\Payment\Gateway\PaymentModule;
use Zend\Http\Headers;
use Zend\Http\PhpEnvironment\Request;
use Zend\Http\Client;
use Zend\Uri\UriFactory;
use Braintree_Gateway;



class BraintreePayment extends PaymentModule
{
    public function getName()
    {
        return 'Braintree';
    }

    public function getImage()
    {
        return 'braintree.png';
    }

    public function getDescription()
    {
        return '';
    }

    public function getHelpText()
    {
        return '
<h3>API Credentials</h3>
<p>You can find Merchant ID, Public Key and Private Key in your Braintree account.</p>
';
    }

    public function setVariables()
    {
        $this
            ->addVariable (
                self::OPTION_DISPLAY_NAME,
                array(
                    'name' => 'Display Name',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => 'Braintree',
                    'required' => true,
                )
            )->addVariable (
                'merchant_id',
                array(
                    'name' => 'Merchant ID (username)',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                'public_key',
                array(
                    'name' => 'Public Key',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                'private_key',
                array(
                    'name' => 'Private Key',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                'google_merchant_id',
                array(
                    'name' => 'Google Merchant Id (for Google Pay)',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => false,
                )
            )->addVariable (
                'apple_pay_store_name',
                array(
                    'name' => 'Store Name (for Apple Pay)',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => false,
                )
            )
            ->addVariable (
                'paypal_enabled',
                array(
                    'name' => 'Enable Paypal',
                    'type' => self::VARIABLE_TYPE_SELECT,
                    'help' => '',
                    'default' => 0,
                    'required' => true,
                    'options' => array (
                        0 => 'No',
                        1 => 'Yes'
                    ),
                )
            )->addVariable (
                'applepay_enabled',
                array(
                    'name' => 'Enable ApplePay',
                    'type' => self::VARIABLE_TYPE_SELECT,
                    'help' => '',
                    'default' => 0,
                    'required' => true,
                    'options' => array (
                        0 => 'No',
                        1 => 'Yes'
                    ),
                )
            )->addVariable (
                'googlepay_enabled',
                array(
                    'name' => 'Enable GooglePay',
                    'type' => self::VARIABLE_TYPE_SELECT,
                    'help' => '',
                    'default' => 0,
                    'required' => true,
                    'options' => array (
                        0 => 'No',
                        1 => 'Yes'
                    ),
                )
            )->addVariable (
                'test_mode',
                array(
                    'name' => 'Test mode',
                    'type' => self::VARIABLE_TYPE_SELECT,
                    'help' => 'The context mode of this module',
                    'default' => 0,
                    'required' => true,
                    'options' => array (
                        0 => 'No',
                        1 => 'Yes'
                    ),
                )
            )->addVariable (
                self::OPTION_ENABLED,
                array(
                    'name' => 'Enabled',
                    'type' => self::VARIABLE_TYPE_SELECT,
                    'help' => '',
                    'default' => 0,
                    'required' => true,
                    'options' => array (
                        0 => 'No',
                        1 => 'Yes'
                    ),
                )
            );
            
    }


    /**
     * @param Order $order
     * @return mixed|void
     */

    public function checkout (Order $order)
    {


        if (!isset($_POST['order_creation']))
        {
        if($this->getOption ('test_mode')){
            $gateway = new Braintree_Gateway([
                'environment' => 'sandbox',
                'merchantId' => $this->getOption ('merchant_id'),
                'publicKey' => $this->getOption ('public_key'),
                'privateKey' => $this->getOption ('private_key')
            ]);
        } else {
            $gateway = new Braintree_Gateway([
                'environment' => 'production',
                'merchantId' => $this->getOption ('merchant_id'),
                'publicKey' => $this->getOption ('public_key'),
                'privateKey' => $this->getOption ('private_key')
            ]);
        }


        $data = array();


        try {
            $clientToken = $gateway->clientToken()->generate();
            $data['client_token'] = $clientToken;
            $data['google_merchant_id'] =
            $data['order_total'] = $order->getTotalPrice();
            $data['currency_code'] = $this->getCurrency()->getCode ();
            $data['apple_pay_store_name'] = $this->getOption ('apple_pay_store_name');
            $data['paypal_enabled'] = $this->getOption ('paypal_enabled');
            $data['applepay_enabled'] = $this->getOption ('applepay_enabled');
            $data['googlepay_enabled'] = $this->getOption ('googlepay_enabled');
            $data ['payment_method'] = $this->getName ();
        } catch (Exception $e) {
            echo 'Caught exception: ' . $e->getMessage() . "\n";
        }
        \Moto\Features::init();
        $twig = \Moto\Render::getInstance ();
        $twig
            ->getLoader ()
            ->addPath (MOTOSTORE_ROOT_DIR . '/extensions/payment_braintree/template/');
        echo $twig->render ('paymentform.twig', $data);

        exit;

        } else  if (isset($_POST['order_creation'])){
            $em = $this->serviceLocator->get ('MotoStore\EntityManager');
            if($this->getOption ('test_mode')){
                $gateway = new Braintree_Gateway([
                    'environment' => 'sandbox',
                    'merchantId' => $this->getOption ('merchant_id'),
                    'publicKey' => $this->getOption ('public_key'),
                    'privateKey' => $this->getOption ('private_key')
                ]);
            } else {
                $gateway = new Braintree_Gateway([
                    'environment' => 'production',
                    'merchantId' => $this->getOption ('merchant_id'),
                    'publicKey' => $this->getOption ('public_key'),
                    'privateKey' => $this->getOption ('private_key')
                ]);
            }
            $clientNonce = ($_POST['authorization_token']);



            try {
                $result = $gateway->transaction()->sale([
                    'amount' => $order->getTotalPrice(),
                    'paymentMethodNonce' => $clientNonce,
                    'options' => [
                        'submitForSettlement' => True
                    ]
                ]);


                if ($result->success == true && $result->transaction){
                    $transaction = new Transaction ();
                    $transaction->setOrder ($order);
                    $transaction->setTransactionId ( $result->transaction->id);
                    $date = new \DateTime ();
                    $transaction->setDate ($date);
                    $transaction->setPaymentMethod($this->getName ());
                    $transactionStatus = Transaction::PAYMENT_STATUS_PAID;
                    $orderStatus = Order::ORDER_PAYMENT_ACCEPTED;
                    \Moto\Hook::trigger("MOTOSTORE_HOOK_ORDER_PAID",$order->getOrderId());
                    $transaction->setStatus ($transactionStatus);
                    $order->setStatus ($orderStatus);
                    $transaction->setAdditional(json_encode ($result));
                    $em->persist ($transaction);
                    $order->addTransaction ($transaction);
                    $em->merge ($order);
                    $em->flush ();
                    $this->getCart ()->setIsSuccess (true);
                }

                    Link::redirect ('checkout', 'success', array ('provider' => $this->getName ()));
                    exit;

            } catch (Exception $e) {
                echo 'Caught exception: ' . $e->getMessage() . "\n";
            }


        }


    }

    /**
     * Handle Success
     *
     * @return mixed
     */
    public function handle (Request $request)
    {

        $order = $this->getCurrentOrder ($request->getPost('reference', 0));
        $em = $this->serviceLocator->get ('MotoStore\EntityManager');
        if ($order)
        {
            $event_type         = $request->getQuery ('event_type', '');
            $transaction_id         = $request->getQuery ('order_id', rand());
            if ($event_type == "ORDER_ACCEPTED"){
                $transaction = new Transaction ();
                $transaction->setOrder ($order);
                $transaction->setTransactionId ($transaction_id);
                $date = new \DateTime ();
                $transaction->setDate ($date);
                $transaction->setPaymentMethod($this->getName ());
                $transactionStatus = Transaction::PAYMENT_STATUS_PAID;
                $orderStatus = Order::ORDER_PAYMENT_ACCEPTED;
                \Moto\Hook::trigger("MOTOSTORE_HOOK_ORDER_PAID",$order->getOrderId());
                $transaction->setStatus ($transactionStatus);
                $order->setStatus ($orderStatus);
                $transaction->setAdditional(json_encode ($request->getQuery ()));
                $em->persist ($transaction);
                $order->addTransaction ($transaction);
                $em->merge ($order);
                $em->flush ();
                $this->getCart ()->setIsSuccess (true);
            }
        }
        exit;
    }


}