<?php

/***********************************************************************************************************************
 * @author: <for.samohin@gmail.com>
 **********************************************************************************************************************/
namespace MotoStore\Payment\Ccbill;

use MotoStore\Content\Helper\Link;
use MotoStore\Order\Entity\Order;
use MotoStore\Order\Entity\Transaction;
use MotoStore\Payment\Gateway\PaymentModule;
use Zend\Http\PhpEnvironment\Request;

class Ccbill extends PaymentModule
{
    public function getName()
    {
        return 'Ccbill';
    }

    public function getImage()
    {
        return 'ccbill.png';
    }

    public function getDescription()
    {
        return 'Please work with your CCBill support representative to activate "Dynamic Pricing" for your account.
You can verify that dynamic pricing is active by selecting "Feature Summary" under the
"Account Info" tab of your CCBill admin menu.  Dynamic pricing status appears at the
bottom of the "Billing Tools" section.<br/><br/>

A "salt" is a string of random data used to make your encryption more secure.
You must contact CCBill Support to generate your salt.  Once set, it will be
visible under the "Advanced" section of the "Sub Account Admin" menu.  It will
appear in the "Encryption Key" field of the "Upgrade Security Setup Information"
section.<br/><br/>

A Form Name is the name of the CCBill form used to collect payment<br/><br/>

Url for notifications: '. Link::storeLink ('checkout', 'success', array ('provider' => $this->getName ()), false) ;
    }

    public function getHelpText()
    {
        return 'Please work with your CCBill support representative to activate "Dynamic Pricing" for your account.
You can verify that dynamic pricing is active by selecting "Feature Summary" under the
"Account Info" tab of your CCBill admin menu.  Dynamic pricing status appears at the
bottom of the "Billing Tools" section.<br/><br/>

A "salt" is a string of random data used to make your encryption more secure.
You must contact CCBill Support to generate your salt.  Once set, it will be
visible under the "Advanced" section of the "Sub Account Admin" menu.  It will
appear in the "Encryption Key" field of the "Upgrade Security Setup Information"
section.<br/><br/>

A Form Name is the name of the CCBill form used to collect payment<br/><br/>

Url for notifications: '. Link::storeLink ('checkout', 'success', array ('provider' => $this->getName ()), false) ;
    }

    public function setVariables()
    {
        $this
            ->addVariable (
                self::OPTION_DISPLAY_NAME,
                array(
                    'name' => 'Display Name',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => 'Ccbill',
                    'required' => true,
                )
            )->addVariable (
                'account_no',
                array(
                    'name' => 'Client Account Number',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                'sub_account_no',
                array(
                    'name' => 'Client SubAccount Number',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                'form_name',
                array(
                    'name' => 'Form Name',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                'salt',
                array(
                    'name' => 'Salt',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                'currency_code',
                array(
                    'name' => 'Currency',
                    'type' => self::VARIABLE_TYPE_SELECT,
                    'help' => '',
                    'default' => 0,
                    'required' => true,
                    'options' => array(
                        '840' => 'USD',
                        '978' => 'EUR',
                        '036' => 'AUD',
                        '124' => 'CAD',
                        '826' => 'GBP',
                        '392' => 'JPY'
                    ),
                )
            )->addVariable (
                self::OPTION_ENABLED,
                array(
                    'name' => 'Enabled',
                    'type' => self::VARIABLE_TYPE_SELECT,
                    'help' => '',
                    'default' => 0,
                    'required' => true,
                    'options' => array (
                        0 => 'No',
                        1 => 'Yes'
                    ),
                )
            );
            
    }
    /**
     * @param Order $order
     * @return mixed|void
     */
    public function checkout(Order $order)
    {
        $cartTotal = '' . number_format($order->getTotalPrice (), 2, '.', '');
        $billingPeriodInDays = 2;
        $salt = $this->getOption ('salt');
        $ccbill_hash = $this->get_digest($cartTotal, $billingPeriodInDays,  $this->getOption ('currency_code'), $salt);
        $ccbill_args = 'clientAccnum='    .  $this->getOption ('account_no')
            . '&clientSubacc='   . $this->getOption ('sub_account_no')
            . '&formName='       . $this->getOption ('form_name')
            . '&formPrice='      . $cartTotal
            . '&formPeriod='     . $billingPeriodInDays
            . '&currencyCode='   . $this->getOption ('currency_code')
            . '&customer_fname=' . $order->getFirstName ()
            . '&customer_lname=' . $order->getLastName ()
            . '&email='          . $order->getEmail ()
            . '&zipcode='        . $order->getPostCode ()
            . '&country='        . ($order->getCountry () ? $order->getCountry ()->getName () : '')
            . '&city='           . $order->getCity ()
            . '&state='          . ($order->getState () ? $order->getState ()->getName () : '')
            . '&address1='       . $order->getAddress1 ()
            . '&wc_orderid='     .  $order->getOrderId()
            . '&formDigest='     . $ccbill_hash;

        header ('Location: ' . 'https://bill.ccbill.com/jpost/signup.cgi?' . $ccbill_args);
        exit;

    }


    public function get_digest($formattedCartTotal, $billingPeriodInDays, $currencyCode, $salt) {

        $stringToHash = '' . $formattedCartTotal
            . $billingPeriodInDays
            . $currencyCode
            . $salt;

        return md5($stringToHash);

    }



    public function handle(Request $request)
    {
        if (isset($_REQUEST['EventType'])){

            $responseAction = isset($_REQUEST['EventType']) ? $_REQUEST['EventType'] : '';

            if(strlen($responseAction) < 1){
                $responseAction = isset($_REQUEST['Action']) ? $_REQUEST['Action'] : '';

                if(strpos($responseAction, 'Approval_Post') !== false)
                    $responseAction = 'approval_post';

            }

            if(strlen($responseAction) < 1 &&
                ((isset($_POST['subscription_id']) && strlen($_POST['subscription_id']) > 0) ||
                    (isset($_POST['subscriptionId']) && strlen($_POST['subscriptionId']) > 0)))
                $responseAction = 'approval_post';

            switch(strToLower($responseAction)){
                case 'checkoutsuccess':
                    die('<p>Thank you for your order.  Your payment has been approved.</p><p><a href="' . Link::storeLink ('checkout', 'success', array ('provider' => $this->getName (), 'motoorderid' => $order->getOrderId ()), false) . '">Return to store</a></p>');
                    break;
                case 'checkoutfailure':
                    die('<p>Unfortunately, your payment was declined.</p><p><a href="' .Link::storeLink ('cart', null, array(), false) . '">Return to Cart</a></p>');
                    break;
                case 'approval_post':
                case 'NewSaleSuccess':  $this->processIpn($request);
                    break;
                case 'denial_post':
                case 'NewSaleFailure':
                    die('Failure');
                    break;
                default: die( "CCBill IPN Request Failure");
                    break;
            }// end switch

            die('Failure');

        }
    }
    public function processIpn(Request $request){
        $p = $_POST;
        $orderNumber = -1;
        $prefix = isset($p['X-wc_orderid']) ? 'X-' : '';
        if(isset($p[$prefix . 'wc_orderid']))
            $orderNumber = $p[$prefix . 'wc_orderid'];
        else
            die('Order not found');
        $success = false;
        if(strlen($prefix) == 0 && isset($p['subscription_id'])){
            $txId = $p['subscription_id'];
            $success = true;
        }
        else if(strlen($prefix) > 0 && isset($p['subscriptionId'])){
            $txId = $p['subscriptionId'];
            $success = true;
        }
        $order = null;

        if($success == true)
        {
            $order = $this->getCurrentOrder ($orderNumber);
            $tCartTotal = -1;
            $tPeriod = -1;

            $formDigest = '';

            if(isset($p[$prefix . 'formPrice'])) {
                $tCartTotal = $p[$prefix . 'formPrice'];
                $tPeriod    = $p[$prefix . 'formPeriod'];
                $formDigest = $p[$prefix . 'formDigest'];
            }
            else if(isset($p['billedInitialPrice'])) {
                $tCartTotal = $p['billedInitialPrice'];
                $tPeriod    = $p['initialPeriod'];
            }// end if/else
            else if(isset($p['initialPrice'])) {
                $tCartTotal = '' . number_format($p['initialPrice'], 2, '.', '');
                $tPeriod    = $p['initialPeriod'];
            }// end if/else

            $tCurrencyCode = isset($p['billedCurrencyCode']) ? $p['billedCurrencyCode'] : '';
            $wCartTotal = '' . number_format($tCartTotal, 2, '.', '');
            $myHash = $this->get_digest($wCartTotal, $tPeriod, $tCurrencyCode, $this->getOption('salt'));
            if(strlen($formDigest) > 0) {
                if($formDigest != $myHash)
                    $success = false;
            }
            else {
                if($wCartTotal != $tCartTotal) {
                    $success = false;
                }
            }
        }

        if($success == true)
        {
            $order = $this->getCurrentOrder ($orderNumber);
            $em = $this->serviceLocator->get ('MotoStore\EntityManager');
            $status = Transaction::PAYMENT_STATUS_PAID;
            $orderStatus = Order::ORDER_PAYMENT_ACCEPTED;
            $transaction = new Transaction ();
            $transaction->setOrder ($order);
            $transaction->setTransactionId ($p['subscription_id']);
            $date = new \DateTime ();
            $transaction->setDate ($date);
            $transaction->setPaymentMethod($this->getName ());
            $transaction->setStatus ($status);
            $transaction->setAdditional(json_encode ($_REQUEST));
            $em->persist ($transaction);
            $order->addTransaction ($transaction);
            $order->setStatus ($orderStatus);
            $em->merge ($order);
            $em->flush ();
            $this->getCart ()->setIsSuccess (true);
            \Moto\Hook::trigger("MOTOSTORE_HOOK_ORDER_PAID",$order->getOrderId());

            die('Success');
        }
        else{
            die('Failure');
        }
    }
}