<?php

/***********************************************************************************************************************
 * @author: <for.samohin@gmail.com>
 **********************************************************************************************************************/
namespace MotoStore\Payment\Epay;

use MotoStore\Content\Helper\Link;
use MotoStore\Order\Entity\Order;
use MotoStore\Order\Entity\Transaction;
use MotoStore\Payment\Gateway\PaymentModule;
use Zend\Http\PhpEnvironment\Request;

class Epay extends PaymentModule
{
    public function getName()
    {
        return 'Epay';
    }

    public function getImage()
    {
        return 'epay.png';
    }

    public function getDescription()
    {
        return 'Find your merchant number in the menu Settings -> Payment system in the administration';
    }

    public function getHelpText()
    {
        return '';
    }

    public function setVariables()
    {
        $this
            ->addVariable (
                self::OPTION_DISPLAY_NAME,
                array(
                    'name' => 'Display Name',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => 'Epay',
                    'required' => true,
                )
            )->addVariable (
                'merchantnumber',
                array(
                    'name' => 'Merchant Number',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                'secret_key',
                array(
                    'name' => 'Secret Key',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                self::OPTION_ENABLED,
                array(
                    'name' => 'Enabled',
                    'type' => self::VARIABLE_TYPE_SELECT,
                    'help' => '',
                    'default' => 0,
                    'required' => true,
                    'options' => array (
                        0 => 'No',
                        1 => 'Yes'
                    ),
                )
            );
            
    }
    /**
     * @param Order $order
     * @return mixed|void
     */
    public function checkout(Order $order)
    {
        $currency = $this->getCurrency();
        $data = array(
            "merchantnumber"  => $this->getOption ('merchantnumber'),
            "orderid"     => $order->getOrderId(),
            "amount"       => $order->getTotalPrice () * $currency->getExchangeRate ()*100,
            "callbackurl" => Link::storeLink ('checkout', 'success', array ('provider' => $this->getName ()), false),
            "accepturl" => Link::storeLink ('checkout', 'success', array ('provider' => $this->getName ()), false),
            "currency" => $currency->getCode (),
            "windowstate" => 3,
            "language" => 0,
        );

        echo $this->form('https://ssl.ditonlinebetalingssystem.dk/integration/ewindow/Default.aspx', $data);


    }

    public function handle(Request $request)
    {
        if (isset($_REQUEST['txnid']) &&isset($_REQUEST['orderid'])){
            $orderid = $_REQUEST['orderid'];
            $trans = $_REQUEST['txnid'];
            $order = $this->getCurrentOrder ($orderid);
            if (!$order){
                die('Order not found');
            }
            $em = $this->serviceLocator->get ('MotoStore\EntityManager');
            $status = Transaction::PAYMENT_STATUS_PAID;
            $orderStatus = Order::ORDER_PAYMENT_ACCEPTED;
            $transaction = new Transaction ();
            $transaction->setOrder ($order);
            $transaction->setTransactionId ($trans);
            $date = new \DateTime ();
            $transaction->setDate ($date);
            $transaction->setPaymentMethod($this->getName ());
            $transaction->setStatus ($status);
            $transaction->setAdditional(json_encode ($_REQUEST));
            $em->persist ($transaction);
            $order->addTransaction ($transaction);
            $order->setStatus ($orderStatus);
            $em->merge ($order);
            $em->flush ();
            $this->getCart ()->setIsSuccess (true);
            \Moto\Hook::trigger("MOTOSTORE_HOOK_ORDER_PAID",$order->getOrderId());
        }
    }
}