<?php

/***********************************************************************************************************************
 * @author: <for.samohin@gmail.com>
 **********************************************************************************************************************/
namespace MotoStore\Payment\Ipay88;

use MotoStore\Content\Helper\Link;
use MotoStore\Order\Entity\Order;
use MotoStore\Order\Entity\Transaction;
use MotoStore\Payment\Gateway\PaymentModule;
use Zend\Http\PhpEnvironment\Request;

class Ipay88 extends PaymentModule
{
    public $hash_amount      = 0;
    public $formatted_amount = 0;

    public function getName()
    {
        return 'Ipay88';
    }

    public function getImage()
    {
        return 'ipay88.png';
    }

    public function getDescription()
    {
        return 'You can find your merchant information in Ipay88 account https://www.mobile88.com/epayment/report/ ';
    }

    public function getHelpText()
    {
        return 'You can find your merchant information in Ipay88 account https://www.mobile88.com/epayment/report/ ';
    }

    public function setVariables()
    {
        $this
            ->addVariable (
                self::OPTION_DISPLAY_NAME,
                array(
                    'name' => 'Display Name',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => 'Ipay88',
                    'required' => true,
                )
            )->addVariable (
                'merchantCode',
                array(
                    'name' => 'Merchant Code',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                'merchantKey',
                array(
                    'name' => 'Merchant Key',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                self::OPTION_ENABLED,
                array(
                    'name' => 'Enabled',
                    'type' => self::VARIABLE_TYPE_SELECT,
                    'help' => '',
                    'default' => 0,
                    'required' => true,
                    'options' => array (
                        0 => 'No',
                        1 => 'Yes'
                    ),
                )
            )->addVariable (
                'test_mode',
                array(
                    'name' => 'Sandbox Mode',
                    'type' => self::VARIABLE_TYPE_SELECT,
                    'help' => '',
                    'default' => 0,
                    'required' => true,
                    'options' => array (
                        0 => 'No',
                        1 => 'Yes'
                    ),
                )
            );
            
    }
    /**
     * @param Order $order
     * @return mixed|void
     */
    public function checkout(Order $order)
    {
        $link = 'https://payment.ipay88.com.ph/epayment/entry.asp';
        if ($this->getOption ('test_mode')){
            $link = 'https://sandbox.ipay88.com.ph/epayment/entry.asp';
        }

        $names = array ();

        foreach ($order->getProducts () as $product)
        {
            $names [] = $product->getName ();
        }
        $this->format_amount( $order->getTotalPrice() );
        $currency = $this->getCurrency();
        $data = array(
            "MerchantCode"  => $this->getOption ('merchantCode'),
            "RefNo"     => $order->getOrderId(),
            "Amount"     => $this->formatted_amount,
            "Currency"     => $currency->getCode(),
            "ProdDesc"     => 'Order #' . $order->getOrderId () . ' [' .  join (', ', $names) . ']',
            'UserName'     => $order->getFullName(),
            'UserEmail'    => $order->getEmail(),
            'UserContact'  => $order->getContactPhone(),
            "ResponseURL" => Link::storeLink ('checkout', 'success', array ('provider' => $this->getName ()), false),
            "BackendURL" => Link::storeLink ('checkout', 'success', array ('provider' => $this->getName ()), false),
        );
        $data['Signature'] = $this->generate_sha256_signature( $data, false );
        $data['SignatureType']= "SHA256";

        echo $this->form($link, $data);
    }


    function validate_response() {
        $signature = $this->generate_sha256_signature( $_POST );
        if ( isset($_POST['Signature']) && $_POST['Signature'] == $signature ) {
            $order = $this->getCurrentOrder ($_REQUEST['RefNo']);
            if (!$order){
                die('Order not found');
            }
            $this->format_amount( $order->getTotalPrice() );
            $order_total = number_format( $order->getTotalPrice(), 2, '', '' );
            if ( abs( $order_total - $this->hash_amount ) == 0 ) {

                return true;
            }
            return false;
        } else {

            return false;
        }
    }

    public function handle(Request $request) {
        $posted = stripslashes( $_POST );
        $is_backend_notification = (isset($_GET['iPay88_response'])&&  $_GET['iPay88_response'] == 'backend' )? true : false;
        $received_ok = 'RECEIVEOK';
        if ( $this->validate_response() ) {
            $refno   = $_POST['RefNo'];
            $transid = $_POST['TransId'];
            $estatus = $_POST['Status'];
            $errdesc = $_POST['ErrDesc'];
            $order = $this->getCurrentOrder ($refno);
            switch ( $estatus ) :
                case 1 : // Successful payment
                    $em = $this->serviceLocator->get ('MotoStore\EntityManager');
                    $status = Transaction::PAYMENT_STATUS_PAID;
                    $orderStatus = Order::ORDER_PAYMENT_ACCEPTED;
                    $transaction = new Transaction ();
                    $transaction->setOrder ($order);
                    $transaction->setTransactionId ($transid);
                    $date = new \DateTime ();
                    $transaction->setDate ($date);
                    $transaction->setPaymentMethod($this->getName ());
                    $transaction->setStatus ($status);
                    $transaction->setAdditional(json_encode ($_REQUEST));
                    $em->persist ($transaction);
                    $order->addTransaction ($transaction);
                    $order->setStatus ($orderStatus);
                    $em->merge ($order);
                    $em->flush ();
                    $this->getCart ()->setIsSuccess (true);
                    \Moto\Hook::trigger("MOTOSTORE_HOOK_ORDER_PAID",$order->getOrderId());
                    break;
                case 2 : // Failed payment
                default :
                    break;
            endswitch;
            // Backend notification will get the OK response
            if ( $is_backend_notification ) {
                echo $received_ok;
                die();
            } else {
                $this->getCart ()->setIsSuccess (true);
            }
        }
    }

    private function generate_sha256_signature( $params, $is_response = true ) {
        $string = '';
        if ( $is_response ) {
            $this->format_amount( str_replace( ',', '', $params['Amount'] ) );
            $string = $params['PaymentId'] . $params['RefNo'] . $this->hash_amount . $params['Currency'] . $params['Status'];
        } else {
            $string = $params['RefNo'] . $this->hash_amount . $params['Currency'];
        }
        $string = $this->getOption ('merchantKey') . $this->getOption ('merchantCode') . $string;
        return $this->iPay88_signature($string );
    }

    public function hex2bin( $hexSource ) {
        $bin = '';
        for ( $i = 0; $i < strlen( $hexSource ); $i = $i + 2 ) {
            $bin .= chr( hexdec( substr( $hexSource, $i, 2 ) ) );
        }
        return $bin;
    }

    public function format_amount( $amount ) {
        if ( is_numeric( $amount ) ) {
            $this->hash_amount      = number_format( $amount, 2, '', '' );
            $this->formatted_amount = number_format( $amount, 2, '.', ',' );
        }
    }

    public function iPay88_signature($source)
    {
        return hash('sha256', $source);
    }
}