<?php

/***********************************************************************************************************************
 * @author: <for.samohin@gmail.com>
 **********************************************************************************************************************/
namespace MotoStore\Payment\KlarnaPayment;

use MotoStore\Content\Helper\Link;
use MotoStore\Order\Entity\Order;
use MotoStore\Order\Entity\Product;
use MotoStore\Order\Entity\Transaction;
use MotoStore\Payment\Gateway\PaymentModule;
use Zend\Http\Headers;
use Zend\Http\PhpEnvironment\Request;
use Zend\Http\Client;
use Zend\Uri\UriFactory;
use Klarna;



class KlarnaPayment extends PaymentModule
{
    public function getName()
    {
        return 'Klarna';
    }

    public function getImage()
    {
        return 'klarna.png';
    }

    public function getDescription()
    {
        return '';
    }

    public function getHelpText()
    {
        return '
<h3>API Credentials</h3>
<p>Make sure that your credentials belong to the right endpoint. If you have credentials for the US ,
 such credentials will not work for the EU  and you will get 401 Unauthorized exception.</p>
';
    }

    public function setVariables()
    {
        $this
            ->addVariable (
                self::OPTION_DISPLAY_NAME,
                array(
                    'name' => 'Display Name',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => 'Klarna',
                    'required' => true,
                )
            )->addVariable (
                'merchant_id',
                array(
                    'name' => 'Merchant ID (username)',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                'shared_secret',
                array(
                    'name' => 'Secret Key',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                'test_mode',
                array(
                    'name' => 'Test mode',
                    'type' => self::VARIABLE_TYPE_SELECT,
                    'help' => 'The context mode of this module',
                    'default' => 0,
                    'required' => true,
                    'options' => array (
                        0 => 'No',
                        1 => 'Yes'
                    ),
                )
            )->addVariable (
                'region',
                array(
                    'name' => 'Region',
                    'type' => self::VARIABLE_TYPE_SELECT,
                    'help' => 'The context mode of this module',
                    'default' => 1,
                    'required' => true,
                    'options' => array (
                        0 => 'US',
                        1 => 'EU'
                    ),
                )
            )->addVariable (
                self::OPTION_ENABLED,
                array(
                    'name' => 'Enabled',
                    'type' => self::VARIABLE_TYPE_SELECT,
                    'help' => '',
                    'default' => 0,
                    'required' => true,
                    'options' => array (
                        0 => 'No',
                        1 => 'Yes'
                    ),
                )
            );
            
    }
    private function getEndpoint(){
        /*
     EU_BASE_URL = 'https://api.klarna.com'
     EU_TEST_BASE_URL = 'https://api.playground.klarna.com'
     NA_BASE_URL = 'https://api-na.klarna.com'
     NA_TEST_BASE_URL = 'https://api-na.playground.klarna.com'
     */
        if ($this->getOption ('test_mode')){
            if ($this->getOption ('region')){
                return Klarna\Rest\Transport\ConnectorInterface::EU_TEST_BASE_URL;
            } else {
                return Klarna\Rest\Transport\ConnectorInterface::NA_TEST_BASE_URL;
            }
        } else {
            if ($this->getOption ('region')){
                return Klarna\Rest\Transport\ConnectorInterface::EU_BASE_URL;
            } else {
                return Klarna\Rest\Transport\ConnectorInterface::NA_TEST_BASE_URL;
            }
        }
    }

    /**
     * @param Order $order
     * @return mixed|void
     */

    public function checkout (Order $order)
    {


        if (!isset($_POST['order_creation']))
        {
        $order_lines = $this->getOrderLines($order);
        $data = array();
        $merchantId = $this->getOption ('merchant_id');
        $sharedSecret = $this->getOption ('shared_secret');
        $apiEndpoint =   $this->getEndpoint();
        $connector = Klarna\Rest\Transport\Connector::create(
            $merchantId,
            $sharedSecret,
            $apiEndpoint
        );
        $country_code = $order->getCountry()? strtolower($order->getCountry()->getIso2()) : 'gb';
        $order_object = [

            "order_id" => $order->getOrderId(),
            "purchase_country" => $country_code,
            "purchase_currency" => strtolower($this->getCurrency()->getCode ()),
            "locale" => "en-gb",
            "order_amount" => intval($order->getTotalPrice()),
            "order_tax_amount" => intval($order->getTaxPrice()),
            "order_lines"=>$order_lines,
        ];

        try {
            $session = new Klarna\Rest\Payments\Sessions($connector);

            $session->create($order_object);
            $sessionId = $session->getId();
            $data['session_id'] = $sessionId;
            $data['client_token'] = $session['client_token'];
            $data['payment_categories'] = $session['payment_method_categories'];
            $data['order_id'] = $order->getOrderId();
            $data ['payment_method'] = $this->getName ();
        } catch (Exception $e) {
            echo 'Caught exception: ' . $e->getMessage() . "\n";
        }
        \Moto\Features::init();
        $twig = \Moto\Render::getInstance ();
        $twig
            ->getLoader ()
            ->addPath (MOTOSTORE_ROOT_DIR . '/extensions/payment_klarna/template/');
        echo $twig->render ('paymentform.twig', $data);

        exit;

        } else  if (isset($_POST['order_creation'])){

            $auth_token = ($_POST['authorization_token']);
            $merchantId = $this->getOption ('merchant_id');
            $sharedSecret = $this->getOption ('shared_secret');
            $apiEndpoint =   $this->getEndpoint();
            $connector = Klarna\Rest\Transport\Connector::create(
                $merchantId,
                $sharedSecret,
                $apiEndpoint
            );
            $order_lines = $this->getOrderLines($order);
            $country_code = $order->getCountry()? strtolower($order->getCountry()->getIso2()) : 'gb';
            $order_object = [
                "order_id" => $order->getOrderId(),
                "purchase_country" => $country_code,
                "purchase_currency" => strtolower($this->getCurrency()->getCode ()),
                "locale" => "en-gb",
                "order_amount" => intval($order->getTotalPrice()),
                "order_tax_amount" => intval($order->getTaxPrice()),
                "order_lines"=>$order_lines,
                "merchant_urls"=>[
                    "terms" => (Link::storeLink ('checkout', 'success', array ('provider' => $this->getName (), 'reference' => $order->getOrderId ()), false)),
                    "checkout" => (Link::storeLink ('checkout', 'confirmation', array (), false)),
                    "confirmation" => (Link::storeLink ('checkout', 'succcess', array (), false)),
                    "push" => (Link::storeLink ('checkout', 'success', array ('provider' => $this->getName (), 'reference' => $order->getOrderId ()), false)),
                    "notification" => (Link::storeLink ('checkout', 'success', array ('provider' => $this->getName (), 'reference' => $order->getOrderId ()), false)),
                ]
            ];


            try {
                $klarna_order = new Klarna\Rest\Payments\Orders($connector, $auth_token);
                $data = $klarna_order->create($order_object);

                if($data['redirect_url']){
                    header ('Location: ' . $data['redirect_url']);
                    exit;
                }  else {
                    Link::redirect ('checkout', 'success', array ('provider' => $this->getName ()));
                    exit;
                }
            } catch (Exception $e) {
                echo 'Caught exception: ' . $e->getMessage() . "\n";
            }


        }


    }

    private function getOrderLines(Order $order) {
        $lines = array(
        );
        foreach ($order->getProducts () as  $product)
        {
            $klarna_item = array(
                "reference" => strval($product->getId()),
                "name" => $product->getName(),
                "quantity" => $product->getQuantity(),
                "unit_price" => intval($product->getPrice()),
                "total_amount" =>  intval($product->getTotalPrice()),

            );
            $lines[] = $klarna_item;
        }
        return $lines;
    }
    /**
     * Handle Success
     *
     * @return mixed
     */
    public function handle (Request $request)
    {

        $order = $this->getCurrentOrder ($request->getPost('reference', 0));
        $em = $this->serviceLocator->get ('MotoStore\EntityManager');
        if ($order)
        {
            $event_type         = $request->getQuery ('event_type', '');
            $transaction_id         = $request->getQuery ('order_id', rand());
            if ($event_type == "ORDER_ACCEPTED"){
                $transaction = new Transaction ();
                $transaction->setOrder ($order);
                $transaction->setTransactionId ($transaction_id);
                $date = new \DateTime ();
                $transaction->setDate ($date);
                $transaction->setPaymentMethod($this->getName ());
                $transactionStatus = Transaction::PAYMENT_STATUS_PAID;
                $orderStatus = Order::ORDER_PAYMENT_ACCEPTED;
                \Moto\Hook::trigger("MOTOSTORE_HOOK_ORDER_PAID",$order->getOrderId());
                $transaction->setStatus ($transactionStatus);
                $order->setStatus ($orderStatus);
                $transaction->setAdditional(json_encode ($request->getQuery ()));
                $em->persist ($transaction);
                $order->addTransaction ($transaction);
                $em->merge ($order);
                $em->flush ();
                $this->getCart ()->setIsSuccess (true);
            }
        }
        exit;
    }


}