<?php

/***********************************************************************************************************************
 * @author: <for.samohin@gmail.com>
 **********************************************************************************************************************/
namespace MotoStore\Payment\Payfast;

use MotoStore\Content\Helper\Link;
use MotoStore\Order\Entity\Order;
use MotoStore\Order\Entity\Transaction;
use MotoStore\Payment\Gateway\PaymentModule;
use Zend\Http\PhpEnvironment\Request;

class Payfast extends PaymentModule
{
    public function getName()
    {
        return 'Payfast';
    }

    public function getImage()
    {
        return 'payfast.png';
    }

    public function getDescription()
    {
        return 'You can find your merchant information in Payfast account ( https://www.payfast.co.za/acc/integration ) ';
    }

    public function getHelpText()
    {
        return 'You can find your merchant information in Payfast account ( https://www.payfast.co.za/acc/integration ) ';
    }

    public function setVariables()
    {
        $this
            ->addVariable (
                self::OPTION_DISPLAY_NAME,
                array(
                    'name' => 'Display Name',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => 'Payfast',
                    'required' => true,
                )
            )->addVariable (
                'merchantnumber',
                array(
                    'name' => 'Merchant ID',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                'secret_key',
                array(
                    'name' => 'Merchant Key',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                'passphrase',
                array(
                    'name' => 'Passphrase',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                self::OPTION_ENABLED,
                array(
                    'name' => 'Enabled',
                    'type' => self::VARIABLE_TYPE_SELECT,
                    'help' => '',
                    'default' => 0,
                    'required' => true,
                    'options' => array (
                        0 => 'No',
                        1 => 'Yes'
                    ),
                )
            )->addVariable (
                'test_mode',
                array(
                    'name' => 'Test Mode',
                    'type' => self::VARIABLE_TYPE_SELECT,
                    'help' => '',
                    'default' => 0,
                    'required' => true,
                    'options' => array (
                        0 => 'No',
                        1 => 'Yes'
                    ),
                )
            );
            
    }
    /**
     * @param Order $order
     * @return mixed|void
     */
    public function checkout(Order $order)
    {
        $link = 'https://payfast.co.za/eng/process';
        if ($this->getOption ('test_mode')){
            $link = 'https://sandbox.payfast.co.za/eng/process';
        }

        $names = array ();

        foreach ($order->getProducts () as $product)
        {
            $names [] = $product->getName ();
        }

        $currency = $this->getCurrency();
        $data = array(
            "merchant_id"  => $this->getOption ('merchantnumber'),
            "merchant_key"  => $this->getOption ('secret_key'),
            "m_payment_id"     => $order->getOrderId(),
            "item_name"     => 'Order #' . $order->getOrderId () . ' [' .  join (', ', $names) . ']',
            "amount"       => $order->getTotalPrice (),
            "notify_url" => Link::storeLink ('checkout', 'success', array ('provider' => $this->getName ()), false),
            "return_url" => Link::storeLink ('checkout', 'success', array ('provider' => $this->getName ()), false),
            "cancel_url" => Link::storeLink ('cart', null, array(), false)
        );

        echo $this->form($link, $data);


    }

    public function handle(Request $request)
    {
        if (isset($_REQUEST['signature']) &&isset($_REQUEST['m_payment_id'])){
            header( 'HTTP/1.0 200 OK' );
            flush();
            $order = $this->getCurrentOrder ($_REQUEST['m_payment_id']);
            if (!$order){
                die('Order not found');
            }
            $pfHost = $this->getOption ('test_mode') ? 'sandbox.payfast.co.za' : 'www.payfast.co.za';
            $pfData = $_POST;
            foreach( $pfData as $key => $val )
            {
                $pfData[$key] = stripslashes( $val );
            }
            $pfData = $_POST;
            $pfParamString = '';
            foreach( $pfData as $key => $val )
            {
                if( $key != 'signature' )
                {
                    $pfParamString .= $key .'='. urlencode( $val ) .'&';
                }
            }
            $pfParamString = substr( $pfParamString, 0, -1 );
            $pfTempParamString = $pfParamString;
            $passPhrase = $this->getOption ('passphrase');
            if( !empty( $passPhrase ) )
            {
                $pfTempParamString .= '&passphrase='.urlencode( $passPhrase );
            }
            $signature = md5( $pfTempParamString );
            if($signature!=$pfData['signature'])
            {
                die('Invalid Signature');
            }

            $cartTotal = $order->getTotalPrice ();
            if( abs( floatval( $cartTotal ) - floatval( $pfData['amount_gross'] ) ) > 0.01 )
            {
                die('Amounts Mismatch');
            }
            $url = 'https://'. $pfHost .'/eng/query/validate';

            $ch = curl_init();
            curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
            curl_setopt( $ch, CURLOPT_HEADER, false );
            curl_setopt( $ch, CURLOPT_SSL_VERIFYHOST, 2 );
            curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, 1 );
            curl_setopt( $ch, CURLOPT_URL, $url );
            curl_setopt( $ch, CURLOPT_POST, true );
            curl_setopt( $ch, CURLOPT_POSTFIELDS, $pfParamString );
            $response = curl_exec( $ch );
            curl_close( $ch );

            $lines = explode( "\r\n", $response );
            $verifyResult = trim( $lines[0] );

            if( strcasecmp( $verifyResult, 'VALID' ) != 0 )
            {
                die('Data not valid');
            }
            $pfPaymentId = $pfData['pf_payment_id'];
            if( $pfData ['payment_status'] == 'COMPLETE' ) {
                $em = $this->serviceLocator->get ('MotoStore\EntityManager');
                $status = Transaction::PAYMENT_STATUS_PAID;
                $orderStatus = Order::ORDER_PAYMENT_ACCEPTED;
                $transaction = new Transaction ();
                $transaction->setOrder ($order);
                $transaction->setTransactionId ($pfPaymentId);
                $date = new \DateTime ();
                $transaction->setDate ($date);
                $transaction->setPaymentMethod($this->getName ());
                $transaction->setStatus ($status);
                $transaction->setAdditional(json_encode ($_REQUEST));
                $em->persist ($transaction);
                $order->addTransaction ($transaction);
                $order->setStatus ($orderStatus);
                $em->merge ($order);
                $em->flush ();
                $this->getCart ()->setIsSuccess (true);
                \Moto\Hook::trigger("MOTOSTORE_HOOK_ORDER_PAID",$order->getOrderId());
            }
        }
    }
}