<?php

/***********************************************************************************************************************
 * @author: <for.samohin@gmail.com>
 **********************************************************************************************************************/
namespace MotoStore\Payment\Paystack;

use MotoStore\Content\Helper\Link;
use MotoStore\Order\Entity\Order;
use MotoStore\Order\Entity\Transaction;
use MotoStore\Payment\Gateway\PaymentModule;
use Zend\Http\PhpEnvironment\Request;

class Paystack extends PaymentModule
{
    public function getName()
    {
        return 'Paystack';
    }

    public function getImage()
    {
        return 'paystack.png';
    }

    public function getDescription()
    {
        return '';
    }

    public function getHelpText()
    {
        return 'Secret Key and Public Key for your website you can get in your <a target="_blank" href="https://dashboard.paystack.com/#/settings/developer">Paystack Account</a>.
<br/><br/> Webhook URL in your Paystack Account need to be set to:<br/><br/>' . Link::storeLink ('payments', null, array ('provider' => $this->getName ()), false) . '
<br/><br/> Please, note, that Paystack accepts payments only in NGN';
    }

    public function setVariables()
    {
        $this
            ->addVariable (
                self::OPTION_DISPLAY_NAME,
                array(
                    'name' => 'Display Name',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => 'Mollie',
                    'required' => true,
                )
            )->addVariable (
                'public_key',
                array(
                    'name' => 'Public Key',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                'secret_key',
                array(
                    'name' => 'Secret Key',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                self::OPTION_ENABLED,
                array(
                    'name' => 'Enabled',
                    'type' => self::VARIABLE_TYPE_SELECT,
                    'help' => '',
                    'default' => 0,
                    'required' => true,
                    'options' => array (
                        0 => 'No',
                        1 => 'Yes'
                    ),
                )
            );
            
    }

    public function checkout(Order $order)
    {
        $currency = $this->getCurrency ();
        $data = array (
            "email"      =>  $order->getEmail(),
            "amount"      => number_format ($order->getTotalPrice () * $currency->getExchangeRate ()*100, 2, '.', ''),
            "reference" =>  $order->getOrderId (),
            "callback_url" => Link::storeLink ('checkout', 'success', array ('provider' => $this->getName ()), false),
        );
        $url = "https://api.paystack.co/transaction/initialize";
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL,$url);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS,json_encode($data));  //Post Fields
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $headers = [
            'Authorization: Bearer '. $this->getOption ('secret_key'),
            'Content-Type: application/json',

        ];
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        $request = curl_exec ($ch);
        curl_close ($ch);
        if ($request) {
            $result = json_decode($request, true);
        }

        if (isset($result) && isset($result['data']) && isset($result['data']['authorization_url'])){
            header('Location: ' . $result['data']['authorization_url']);
            exit;
        } else {
            Link::redirect ('checkout', 'success');
        }

    }

    public function handle(Request $request)
    {
        if (isset($_GET['reference'])){
            $this->checkTransaction($_GET['reference']);

        } else {
            $body = @file_get_contents("php://input");
            $signature = (isset($_SERVER['HTTP_X_PAYSTACK_SIGNATURE']) ? $_SERVER['HTTP_X_PAYSTACK_SIGNATURE'] : '');
            if (!$signature) {

            }
            if( $signature !== hash_hmac('sha512', $body, $this->getOption ('secret_key')) ){
                exit();
            }
            http_response_code(200);
            $event = json_decode($body);
            switch($event->event){
                // charge.success
                case 'charge.success':
                    $this->checkTransaction($event->data['reference']);
                    break;
            }
            exit();
        }


    }
    private function checkTransaction ($orderId)
    {
        $curl = curl_init();
        if(!$orderId){
            die('No order supplied');
        }
        curl_setopt_array($curl, array(
            CURLOPT_URL => "https://api.paystack.co/transaction/verify/" . rawurlencode($orderId),
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_HTTPHEADER => [
                "accept: application/json",
                "authorization: Bearer ". $this->getOption ('secret_key'),
                "cache-control: no-cache"
            ],
        ));
        $response = curl_exec($curl);
        $err = curl_error($curl);
        if($err){
            die('Curl returned error: ' . $err);
        }
        $tranx = json_decode($response);
        if(!$tranx->status){
            // there was an error from the API
            die('API returned error: ' . $tranx->message);
        }
        if('success' == $tranx->data->status){
            $order = $this->getCurrentOrder ($orderId);
            if (!$order){
                die('Order not found');
            }
            $em = $this->serviceLocator->get ('MotoStore\EntityManager');
            $status = Transaction::PAYMENT_STATUS_PAID;
            $orderStatus = Order::ORDER_PAYMENT_ACCEPTED;
            \Moto\Hook::trigger("MOTOSTORE_HOOK_ORDER_PAID",$order->getOrderId());
            $transaction = new Transaction ();
            $transaction->setOrder ($order);
            $transaction->setTransactionId ($tranx->data->id);
            $date = \DateTime::createFromFormat('Y-m-d\TH:i:s\.\0\0\0\Z',$tranx->data->created_at);
            $transaction->setDate ($date);
            $transaction->setPaymentMethod($this->getName ());
            $transaction->setStatus ($status);
            $transaction->setAdditional(json_encode ($tranx->data));
            $em->persist ($transaction);
            $order->addTransaction ($transaction);
            $order->setStatus ($orderStatus);
            $em->merge ($order);
            $em->flush ();
            $this->getCart ()->setIsSuccess (true);
        }
    }


}