<?php

/***********************************************************************************************************************
 * @author: <for.samohin@gmail.com>
 **********************************************************************************************************************/
namespace MotoStore\Payment\Squareup;

use MotoStore\Content\Helper\Link;
use MotoStore\Order\Entity\Order;
use MotoStore\Order\Entity\Transaction;
use MotoStore\Payment\Gateway\PaymentModule;
use Zend\Http\PhpEnvironment\Request;
use SquareConnect;

class Squareup extends PaymentModule
{


    public function getName()
    {
        return 'Squareup';
    }

    public function getImage()
    {
        return 'square.png';
    }

    public function getDescription()
    {
        return '';
    }

    public function getHelpText()
    {
        return 'Access Tokens and Location ID for your website you can get in your App settings in <a href="https://squareup.com/dashboard/apps/my-apps" target="_blank">Square Account</a>';
    }

    public function setVariables()
    {
        $this
            ->addVariable (
                self::OPTION_DISPLAY_NAME,
                array(
                    'name' => 'Display Name',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => 'Square',
                    'required' => true,
                )
            )->addVariable (
                'access_token',
                array(
                    'name' => 'Live Access Token',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable('sb_access_token',
                array(
                    'name' => 'SandBox Access Token',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                'location_id',
                array(
                    'name' => 'Location ID',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                'sb_mode',
                array(
                    'name' => 'SandBox mode',
                    'type' => self::VARIABLE_TYPE_SELECT,
                    'help' => 'The context mode of this module',
                    'default' => 0,
                    'required' => true,
                    'options' => array (
                        0 => 'No',
                        1 => 'Yes'
                    ),
                )
            )->addVariable (
                self::OPTION_ENABLED,
                array(
                    'name' => 'Enabled',
                    'type' => self::VARIABLE_TYPE_SELECT,
                    'help' => '',
                    'default' => 0,
                    'required' => true,
                    'options' => array (
                        0 => 'No',
                        1 => 'Yes'
                    ),
                )
            );
            
    }

    /**
     * @param Order $order
     * @return mixed|void
     */
    public function checkout(Order $order)
    {
        $currency = $this->getCurrency ();

        $access_token =  ($this->getOption ('sb_mode') == 0)? $this->getOption('access_token') : $this->getOption('sb_access_token');
        $location_id  =  $this->getOption('location_id');

        $defaultApiConfig = new \SquareConnect\Configuration();
        $defaultApiConfig->setAccessToken($access_token);
        $defaultApiClient = new \SquareConnect\ApiClient($defaultApiConfig);
        $checkoutClient = new SquareConnect\Api\CheckoutApi($defaultApiClient);

        $price = new \SquareConnect\Model\Money;
        $price->setAmount( intval($order->getTotalPrice () * $currency->getExchangeRate ()* 100));
        $price->setCurrency($currency->getCode());
        foreach ($order->getProducts () as $product)
        {
            $names [] = $product->getName ();
        }
        $lineItems = array();
        $item  = new \SquareConnect\Model\CreateOrderRequestLineItem;
        $item->setName('Order #' . $order->getOrderId () . ' [' .  join (', ', $names) . ']');
        $item->setQuantity('1');
        $item->setBasePriceMoney($price);

        array_push($lineItems, $item);

        $square_order = new \SquareConnect\Model\CreateOrderRequest();
        $square_order->setIdempotencyKey($order->getOrderId ());
        $square_order->setLineItems($lineItems);

        $checkout = new \SquareConnect\Model\CreateCheckoutRequest();
        $checkout->setIdempotencyKey(uniqid());
        $checkout->setOrder($square_order);
        $checkout->setPrePopulateBuyerEmail($order->getEmail());
        $checkout->setRedirectUrl( Link::storeLink ('checkout', 'success', array ('provider' => $this->getName (), 'motoorderid' => $order->getOrderId ()), false));
        try {
            $result = $checkoutClient->createCheckout(
                $location_id,
                $checkout
            );
            $checkoutUrl = $result->getCheckout()->getCheckoutPageUrl();
            header ('Location: ' . $checkoutUrl);
        } catch (Exception $e) {

        }

    }

    public function handle(Request $request)
    {

        if(isset($_GET["transactionId"]) && isset($_GET["motoorderid"])){
            $order = $this->getCurrentOrder ($request->getQuery ('motoorderid'));
            $access_token =  ($this->getOption ('sb_mode') == 0)? $this->getOption('access_token') : $this->getOption('sb_access_token');
            $location_id  =  $this->getOption('location_id');
            $returnedTransactionId = $_GET["transactionId"];
            $defaultApiConfig = new \SquareConnect\Configuration();
            $defaultApiConfig->setAccessToken($access_token);
            $defaultApiClient = new \SquareConnect\ApiClient($defaultApiConfig);
            $transactionClient = new \SquareConnect\Api\TransactionsApi($defaultApiClient);

            try {

                $apiResponse = $transactionClient->retrieveTransaction(
                    $location_id,
                    $returnedTransactionId
                );
                if (!empty($apiResponse))
                {
                    $apiResponse = json_decode($apiResponse,true);
                    $gotTransaction = $apiResponse['transaction'];
                    if ($gotTransaction['card_details']['status'] == 'CAPTURED'){
                        $em = $this->serviceLocator->get ('MotoStore\EntityManager');

                        $transaction = new Transaction();
                        $transaction->setOrder ($order);
                        $transaction->setTransactionId ($gotTransaction['id']);

                        $date = new \DateTime ();

                        $transaction->setDate ($date);
                        $transaction->setPaymentMethod($this->getName ());

                        $transaction->setStatus (Transaction::PAYMENT_STATUS_PAID);
                        $order->setStatus (Order::ORDER_PAYMENT_ACCEPTED);
                        $this->getCart ()
                            ->setIsSuccess (true);
                        \Moto\Hook::trigger("MOTOSTORE_HOOK_ORDER_PAID",$order->getOrderId());

                        $transaction->setAdditional(json_encode ($apiResponse));
                        $em->persist ($transaction);
                        $order->addTransaction ($transaction);
                        $em->merge ($order);
                        $em->flush ();

                    }

                }



            } catch (Exception $e) {


            }
        }


    }

}