<?php

/***********************************************************************************************************************
 * @author: <for.samohin@gmail.com>
 **********************************************************************************************************************/
namespace MotoStore\Payment\Tinkoff;

use MotoStore\Content\Helper\Link;
use MotoStore\Order\Entity\Order;
use MotoStore\Order\Entity\Transaction;
use MotoStore\Payment\Gateway\PaymentModule;
use Zend\Http\PhpEnvironment\Request;
use MotoStore\Extentions\TinkoffApi;

class Tinkoff extends PaymentModule
{
    public function getName()
    {
        return 'Tinkoff';
    }

    public function getImage()
    {
        return 'tinkoff.png';
    }

    public function getDescription()
    {
        return 'Терминал и Пароль указаны в Личном кабинете https://oplata.tinkoff.ru';
    }

    public function getHelpText()
    {
        return '';
    }

    public function setVariables()
    {
        $this
            ->addVariable (
                self::OPTION_DISPLAY_NAME,
                array(
                    'name' => 'Название метода оплаты',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => 'Tinkoff',
                    'required' => true,
                )
            )->addVariable (
                'merchant_id',
                array(
                    'name' => 'Терминал',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                'secret_key',
                array(
                    'name' => 'Пароль',
                    'type' => self::VARIABLE_TYPE_TEXT,
                    'help' => '',
                    'default' => '',
                    'required' => true,
                )
            )->addVariable (
                self::OPTION_ENABLED,
                array(
                    'name' => 'Enabled',
                    'type' => self::VARIABLE_TYPE_SELECT,
                    'help' => '',
                    'default' => 0,
                    'required' => true,
                    'options' => array (
                        0 => 'No',
                        1 => 'Yes'
                    ),
                )
            );

    }
    /**
     * @param Order $order
     * @return mixed|void
     */
    public function checkout(Order $order)
    {
        $description = '';
        foreach ($order->getProducts () as $product) {
            $description .= $product -> getName();
            $description .= "; ";
        }
        if (strlen($description) > 250) {
            $description = mb_substr($description, 0, 247) . '...';
        }

        $fields = array(
            'OrderId' => $order->getOrderId(),
            'Amount' => round($order->getTotalPrice() * 100),
            'Description' => $description,
            'DATA' => array('Email' => $order->getEmail (), 'Connection_type' => 'MotoCMS',),
        );

        $Tinkoff = new \MotoStore\Extentions\TinkoffApi\TinkoffMerchantAPI($this->getOption('merchant_id'), $this->getOption('secret_key'));
        $response = $Tinkoff->buildQuery('Init', $fields);
        $response = json_decode($response);
        if (isset($response->PaymentURL))
        {
            header ('Location: ' . $response->PaymentURL);
            exit;
        }
        else
        {
            error_log ('Tinkoff Err Request: ' . json_encode($fields));
            error_log ('Tinkoff Err Response: ' . json_encode($response));
            Link::redirect ('checkout', 'success');
        }


    }

    public function handle(Request $request)
    {

        if (isset($_REQUEST['tinkoff_success']) && $_REQUEST['tinkoff_success'] == 1) {
            $order = $this->getCurrentOrder ();
            if (!$order){
                die('Order not found');
            }
            $em = $this->serviceLocator->get ('MotoStore\EntityManager');
            $status = Transaction::PAYMENT_STATUS_PAID;
            $orderStatus = Order::ORDER_PAYMENT_ACCEPTED;
            $transaction = new Transaction ();
            $transaction->setOrder ($order);
            $transaction->setTransactionId ($order->getOrderId());
            $date = new \DateTime ();
            $transaction->setDate ($date);
            $transaction->setPaymentMethod($this->getName ());
            $transaction->setStatus ($status);
            $transaction->setAdditional(json_encode ($_REQUEST));
            $em->persist ($transaction);
            $order->addTransaction ($transaction);
            $order->setStatus ($orderStatus);
            $em->merge ($order);
            $em->flush ();
            $this->getCart ()->setIsSuccess (true);
            \Moto\Hook::trigger("MOTOSTORE_HOOK_ORDER_PAID",$order->getOrderId());
        }

    }

    public function send_data($order)
    {


        return $arrFields;
    }
}