<?php

namespace MotoStore\Extentions\Recaptcha;

use Moto;
use MotoStore;
use MotoStore\Settings\Entity\Option;
use MotoStore\Content\Content\DI\SMStatic;
use MotoStore\Settings\Repository\OptionRepository;


class Recaptcha
{

    public static function displayCaptcha($errors = array()){
        $optRepository = clone SMStatic::getInstance()
            ->get ('MotoStore\EntityManager')
            ->getRepository ('MotoStore\Settings\Entity\Option');
        $enabled =  $optRepository->getOption (Option::OPTION_STORE_SETTINGS_EXTENTIONS, 'enable_recaptcha');
        $sitekey =  $optRepository->getOption (Option::OPTION_STORE_SETTINGS_EXTENTIONS, 'recaptcha_sitekey');
        if (!$enabled || empty($enabled) || empty($sitekey)){
            return '';
        }
        $captcha ='
 <div class="moto-widget-contact_form-group">
        <div id="recaptcha_container" data-sitekey="" style="display: flex;justify-content: center;"></div>';
        if (isset($errors['captcha'])){
            $captcha .= '<span class="moto-widget-contact_form-field-error">Invalid captcha input</span>';
        }
        $captcha .= ' </div>
        <script type="text/javascript">
              var verifyCallback = function(response) {
                alert(response);
              };
              var widget;
              var onloadCallback = function() {
                widget = grecaptcha.render(\'recaptcha_container\', {
                  \'sitekey\' : \''. $sitekey .'\',
                });
            
              };
            </script>
        <script src="https://www.google.com/recaptcha/api.js?onload=onloadCallback&render=explicit" async defer></script>';
        return $captcha;
    }

    public static function checkCaptcha($request){
        $optRepository = clone SMStatic::getInstance()
            ->get ('MotoStore\EntityManager')
            ->getRepository ('MotoStore\Settings\Entity\Option');
        $enabled =  $optRepository->getOption (Option::OPTION_STORE_SETTINGS_EXTENTIONS, 'enable_recaptcha');
        $secretkey =  $optRepository->getOption (Option::OPTION_STORE_SETTINGS_EXTENTIONS, 'recaptcha_secret');
        if (!$enabled || empty($enabled) || empty($secretkey)){
            return false;
        }
        if(!isset($request['g-recaptcha-response'])){
            return array(
                'success'=>false,
                'errors' => array('captcha'=>true)
            );
        }
        $gr_response = $request['g-recaptcha-response'];
        $ip = MotoStore\Extentions\Recaptcha\Recaptcha::getIPv4();
        $url = 'https://www.google.com/recaptcha/api/siteverify?secret='.$secretkey.'&remoteip=' . $ip . '&response='.$gr_response;
        $remote_get = MotoStore\Extentions\Recaptcha\Request::executeRequest($url, '', "GET");
        if ( isset($remote_get['result']) && isset($remote_get['result']['success']) && $remote_get['result']['success'] == true){
            return array(
                'success'=> true
            );
        }  else {
            return array(
                'success'=>false,
                'errors' => array('captcha'=>true)
            );
        }

    }

    public static function getIPv4(){
        $proxy_header = "HTTP_X_FORWARDED_FOR";
        if (array_key_exists ($proxy_header, $_SERVER)) {
            $proxy_list = explode (",", $_SERVER[$proxy_header]);
            $client_ip = false;
            foreach ($proxy_list as $item){
                $ip = trim ($item);
                if (filter_var ($ip, FILTER_VALIDATE_IP,FILTER_FLAG_IPV4)) {
                    $client_ip = $ip;
                }
            }
            if ($client_ip) {
                return $client_ip;
            }
        }
        if (!isset ($_SERVER['REMOTE_ADDR'])) {
            return NULL;
        }
        return $_SERVER['REMOTE_ADDR'];
    }
}